package com.slodon.b2b2c.model.business;

import com.slodon.b2b2c.business.example.PayTxExample;
import com.slodon.b2b2c.business.pojo.PayTx;
import com.slodon.b2b2c.core.constant.OrderConst;
import com.slodon.b2b2c.core.exception.MallException;
import com.slodon.b2b2c.core.response.PagerInfo;
import com.slodon.b2b2c.core.util.AssertUtil;
import com.slodon.b2b2c.dao.read.business.PayTxReadMapper;
import com.slodon.b2b2c.dao.write.business.PayTxWriteMapper;
import com.slodon.b2b2c.starter.util.ZjPayUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.util.List;

/**
 * 订单支付流水号表，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单model
 */
@Component
@Slf4j
public class PayTxModel {
    @Resource
    private PayTxReadMapper payTxReadMapper;

    @Resource
    private PayTxWriteMapper payTxWriteMapper;

    /**
     * 新增订单支付流水号表，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单
     *
     * @param payTx 订单支付流水号信息，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单
     * @return
     */
    public Integer savePayTx(PayTx payTx) {
        int count = payTxWriteMapper.insert(payTx);
        if (count == 0) {
            throw new MallException("添加订单支付流水号表，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单失败，请重试");
        }
        return count;
    }

    /**
     * 根据txId删除订单支付流水号表，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单
     *
     * @param txId txId
     * @return
     */
    public Integer deletePayTx(Integer txId) {
        if (StringUtils.isEmpty(txId)) {
            throw new MallException("请选择要删除的数据");
        }
        int count = payTxWriteMapper.deleteByPrimaryKey(txId);
        if (count == 0) {
            log.error("根据txId：" + txId + "删除订单支付流水号表，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单失败");
            throw new MallException("删除订单支付流水号表，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单失败,请重试");
        }
        return count;
    }

    /**
     * 根据txId更新订单支付流水号表，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单
     *
     * @param payTx 订单支付流水号信息，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单
     * @return
     */
    public Integer updatePayTx(PayTx payTx) {
        if (StringUtils.isEmpty(payTx.getTxId())) {
            throw new MallException("请选择要修改的数据");
        }
        int count = payTxWriteMapper.updateByPrimaryKeySelective(payTx);
        if (count == 0) {
            log.error("根据txId：" + payTx.getTxId() + "更新订单支付流水号表，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单失败");
            throw new MallException("更新订单支付流水号表，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单失败,请重试");
        }
        return count;
    }

    /**
     * 根据txId获取订单支付流水号表，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单详情
     *
     * @param txId txId
     * @return
     */
    public PayTx getPayTxByTxId(Integer txId) {
        return payTxReadMapper.getByPrimaryKey(txId);
    }

    /**
     * 根据条件获取订单支付流水号表，用于中金支付，生成新的流水号之前，要关闭所有支付中的支付订单列表
     *
     * @param example 查询条件信息
     * @param pager   分页信息
     * @return
     */
    public List<PayTx> getPayTxList(PayTxExample example, PagerInfo pager) {
        List<PayTx> payTxList;
        if (pager != null) {
            pager.setRowsCount(payTxReadMapper.countByExample(example));
            payTxList = payTxReadMapper.listPageByExample(example, pager.getStart(), pager.getPageSize());
        } else {
            payTxList = payTxReadMapper.listByExample(example);
        }
        return payTxList;
    }

    /**
     * 保存流水号，并关闭已该订单已生成的流水订单
     *
     * @param paySn 支付单号
     * @param txSn 新流水号
     * @param institutionId 中金支付机构号
     */
    public void closeAndSaveTx(String paySn, String txSn,String institutionId) throws Exception {
        //查询待支付的流水号
        PayTxExample payTxExample = new PayTxExample();
        payTxExample.setPaySn(paySn);
        payTxExample.setStateNotEquals(OrderConst.ZJ_PAY_TX_STATE_2);//过滤已关闭的
        List<PayTx> payTxList = payTxWriteMapper.listByExample(payTxExample);
        if (!CollectionUtils.isEmpty(payTxList)){
            //有需要关闭的流水
            for (PayTx payTx : payTxList) {
                AssertUtil.isTrue(payTx.getState().equals(OrderConst.ZJ_PAY_TX_STATE_3), "订单已支付");//回调成功订单已支付

                //如果未回调成功，实际已支付成功，主动查询支付状态
                Integer state = ZjPayUtil.queryOrderState(payTx.getTxSn(), institutionId);
                AssertUtil.isTrue(state.equals(OrderConst.ZJ_PAY_TX_STATE_3),"订单已支付，无需重复支付");

                //订单未支付，关闭三方支付订单
                ZjPayUtil.closeOrder(payTx.getTxSn(), institutionId);
            }
            //修改数据库
            PayTx update = new PayTx();
            update.setState(OrderConst.ZJ_PAY_TX_STATE_2);
            int count = payTxWriteMapper.updateByExampleSelective(update, payTxExample);
            AssertUtil.isTrue(count == 0,"更新支付流水失败");
        }

        //保存此次流水
        PayTx payTx = new PayTx();
        payTx.setTxSn(txSn);
        payTx.setPaySn(paySn);
        payTx.setState(OrderConst.ZJ_PAY_TX_STATE_1);
        int count = payTxWriteMapper.insert(payTx);
        AssertUtil.isTrue(count == 0,"保存支付流水失败");
    }
}